@@ -5,7 +5,7 @@ use strict;
 use warnings;
 use Carp;
 
-our $VERSION = '0.8';
+our $VERSION = '0.90';
 
 sub AUTOLOAD {
         my $self = shift;
@@ -27,14 +27,27 @@ sub new {
 			};
 	$self = bless $self, $class;
 
-	$self->read($data);
-	return $self;
+	return $self->read($data);
 }
 
 sub trapname {
 	my $self = shift;
 	my $trapname = $self->data->{"SNMPv2-MIB::snmpTrapOID"};
-	return $trapname;
+	return $trapname || undef;
+}
+
+sub packet {
+	my $self = shift;
+	if ($_[0] && ref \$_[0] eq "SCALAR") {
+		return $self->{packet} = shift;
+	}
+	my $opts;
+	$opts = shift if (ref $_[0] eq "HASH");
+	$_ = $self->{packet};
+	if ($opts->{hide_passwords}) {
+		s/\nSNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "(.*?)"\n/\nSNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "*****"\n/;
+	}
+	return $_;
 }
 
 sub expand {
@@ -52,6 +65,8 @@ sub expand {
 			$newval = join(" ", map {"$_=".$self->data->{$_}} (sort keys %h) );
 		} elsif ($key eq "TRAPNAME") {
 			$newval = $self->trapname;
+		} elsif ($key eq "HOSTIP") {
+			$newval = $self->hostip;
 		} else {
 			$newval = $self->data->{$key} || "(null)";
 		}
@@ -77,16 +92,22 @@ sub read {
 	my ($self, $data) = @_;
 	if (ref \$data eq "GLOB") {
 		local $/=""; 
-		$self->{packet} = <$data>;
+		$self->{packet} = <$data> || return undef;
 	} elsif (ref \$data eq "REF") {
 		$self->{packet} = $$data;
 	} else {
 		croak "Bad ref";
 	}
+	$self->{packet} =~ s/\n*$//;
 	my @packet = split("\n", $self->{packet});
-	chomp($_ = $packet[0]);
+	chomp($_ = shift @packet);
 	$self->hostname($_);
-	chomp($_ = $packet[1]);
+	chomp($_ = shift @packet);
+
+	# Extra stuff around the IP packet in Net-SNMP 5.2.1
+	s/^.*\[//;
+	s/\].*$//;
+
 	$self->hostip($_);
 	foreach $_ (@packet) {
 		my ($key, $value) = /^([^ ]+) (.+)$/;
@@ -97,6 +118,7 @@ sub read {
 		}
 		$self->data->{"$key"} = $value;
 	}
+	return $self;
 }
 
 sub fully_translated {
@@ -113,6 +135,10 @@ sub _get_line {
 	$line--;	# Index begins with 1
 	my @packet = split("\n", $self->{packet});
 	$_ = $packet[$line];
+	# Return complete line if requesting P1 or P2
+	if ($line == 0 or $line == 1) {
+		return ($_, undef);
+	}
 	my ($key, undef, $value) = /^([^ ]+)( (.+))?$/;
 	$key = $self->cleanup_string($key);
 	$value = $self->cleanup_string($value) if $value;
@@ -144,22 +170,55 @@ SNMP::Trapinfo - Reading an SNMP trap from Net-SNMP's snmptrapd
   use SNMP::Trapinfo;
   $trap = SNMP::Trapinfo->new(*STDIN);
 
-  print "From host: ", $trap->hostname, $/;
-  print "Trapname: ", $trap->trapname, $/;
-  print "Packet: ", $trap->packet, $/;
-  print "Message: ", $trap->expand('Interface ${V5} is down'), $/;
+  open F, ">> /tmp/trap.log";
+  print F $trap->packet;
+  close F;
+
+  if (! defined $trap->trapname) {
+    die "No trapname in packet";
+  } elsif ($trap->trapname eq "IF-MIB::linkUp" or $trap->trapname eq "IF-MIB::linkDown") {
+    # $mailer is a Mail::Mailer object, for example
+    print $mailer "Received trap :", $trap->trapname, $/,
+      "From host: ", $trap->hostname, $/,
+      "Message: ", $trap->expand('Interface ${V5} received ${TRAPNAME}'), $/;
+  } else {
+    # not expected trap
+  }
 
 =head1 DESCRIPTION
 
 This module allows the user to get to the useful parts of an snmptrapd
-packet. This assumes that you are using the Net-SNMP package from
-http://www.net-snmp.org and that you are letting snmptrapd do all the
-OID translations (no -On option for snmptrapd) and that hostnames are resolved
-itself (no -n option).
+packet, as provided by the Net-SNMP software (http://www.net-snmp.org). 
+You can then take whatever action with the packet, such as sending
+an email, post an IM or passing it to Nagios (http://www.nagios.org).
 
 The most useful method is expand, which evaluates macros based on the packet,
 for your custom messages.
 
+=head1 IMPLEMENTATION
+
+=over 4
+
+=item 1
+
+Create your perl script (such as the example above).
+
+=item 2
+
+Edit snmptrapd.conf so that the default traphandle calls your perl script.
+
+=item 3
+
+Startup snmptrapd and let it do all the OID translations (no -On option) and let it
+do hostname translations (no -n option).
+
+=item 4
+
+Create a trap and check that it has been received and processed
+correctly.
+
+=back
+
 =head1 METHODS
 
 =over 4
@@ -172,7 +231,7 @@ about this packet. An example packet is:
   cisco2611.lon.altinity
   192.168.10.20
   SNMPv2-MIB::sysUpTime.0 9:16:47:53.80
-  SNMPv2-MIB::snmpTrapOID.0 IF-MIB::linkUp.1
+  SNMPv2-MIB::snmpTrapOID.0 IF-MIB::linkUp
   IF-MIB::ifIndex.2 2
   IF-MIB::ifDescr.2 Serial0/0
   IF-MIB::ifType.2 ppp
@@ -181,6 +240,11 @@ about this packet. An example packet is:
   SNMP-COMMUNITY-MIB::snmpTrapCommunity.0 "public"
   SNMPv2-MIB::snmpTrapEnterprise.0 SNMPv2-SMI::enterprises.9.1.186
 
+Any trailing linefeeds will be stripped.
+
+Can specify multiple packets - keep calling SNMP::Trapinfo->new(*STDIN). Will receive an undef if 
+there are no more packets to read.
+
 =item SNMP::Trapinfo->new(\$data)
 
 Instead of a filehandle, can specify a scalar reference that holds the packet data.
@@ -192,14 +256,16 @@ resolved by snmptrapd.
 
 =item hostip
 
-Returns the 2nd line of the packet, which should be the ip address
-of the originating packet.
+Returns the IP address in the 2nd line of the packet, which should be the 
+originating host.
 
 =item trapname
 
 Returns the value of the parameter SNMPv2-MIB::snmpTrapOID. In the
-example above, this method would return IF-MIB::linkUp. The .1 is removed,
-but if it was .1.1, this would not be removed.
+example above, this method would return IF-MIB::linkUp. 
+
+If the SNMPv2-MIB::snmpTrapOID is not found, then will return undef.
+This could mean that the MIB for snmpTrapOID has not been loaded.
 
 =item fully_translated
 
@@ -207,6 +273,11 @@ Returns 0 if the trapname has more than 1 set of trailing digits
 (a single .\d+ would be removed automatically) - this would mean that a
 MIB is missing. Otherwise returns 1.
 
+=item packet( {hide_passwords => 1} )
+
+Returns a scalar with the full packet, as originally received. If hide_passwords is specified,
+will replace the value of snmpTrapCommunity.0 with 5 asterisks.
+
 =item data
 
 Returns a hash ref where the keys consist of the SNMP parameter and
@@ -218,7 +289,7 @@ trap above, a Data::Dumper of $trap->data would give:
           'SNMP-COMMUNITY-MIB::snmpTrapAddress' => '192.168.10.20',
           'IF-MIB::ifType' => 'ppp',
           'IF-MIB::ifIndex' => '2',
-          'SNMPv2-MIB::snmpTrapOID' => 'IF-MIB::linkUp.1',
+          'SNMPv2-MIB::snmpTrapOID' => 'IF-MIB::linkUp',
           'IF-MIB::ifDescr' => 'Serial0/0',
           'SNMP-COMMUNITY-MIB::snmpTrapCommunity' => '"public"',
           'SNMPv2-MIB::sysUpTime' => '9:16:47:53.80',
@@ -246,6 +317,10 @@ ${TRAPNAME} - Returns the trapname (as called from $trap->trapname)
 
 =item *
 
+${HOSTIP} - Returns the IP of the originating packet
+
+=item *
+
 ${IF-MIB::ifType} - Returns the value for the specified parameter. 
 
 =item *
@@ -264,14 +339,24 @@ this would return:
 
 =back
 
-=head1 SEE ALSO
+=head1 VERSION NUMBERING
 
-Net-SNMP - http://www.net-snmp.org
+After a brief flirtation with 3 digit version numbering, I've changed back to X.YY 
+format as perlmodstyle recommends.
+
+=head1 REFERENCES
+
+Net-SNMP - http://www.net-snmp.org. This module has been tested on versions 
+5.1.2 and 5.2.1.
 
 =head1 AUTHOR
 
 Ton Voon, E<lt>ton.voon@altinity.comE<gt>
 
+=head1 CREDITS
+
+Thanks to Brand Hilton for documentation suggestions.
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2006 by Altinity Limited
